# $Id: Dotty.pm,v 1.1 2005/02/22 18:39:55 lordp Exp $


package GOGraph::Dotty::Dotty;

=head1 NAME

  GOGraph::Dotty::Dotty

=head1 DESCRIPTION

  Utility functions for producing Dotty diagrams from GO graphs.
  This class is now also present in the GO::Dotty package, where
  a number of improvements have been made, including supporting
  US spelling, which should save wearing your "u" key out. So
  basically use that one instead.

=cut

use strict;
use Exporter;
use GraphViz;
my @ISA=qw{Exporter};



my %labellednodes;

sub graphviz_to_dotty{
  my $graphviz = shift;

  open( DOTTY, "|dotty -" );
  print DOTTY $graphviz->as_text;
  close( DOTTY );
}

sub go_graph_to_graphviz{
  my $graph = shift;
  my $it = $graph->create_iterator;
  my %relationships;
  my %labellednodes;

  my $graphviz = GraphViz->new(rankdir=>1);

  while (my $ni = $it->next_node_instance ){
    my $term = $ni->term;
    my $parents = $term->get_parent_terms;
    _graphviz_label( $term, $graph,
                     \%labellednodes, $graphviz );

    foreach my $p (@$parents){
      _graphviz_label( $p, $graph->is_focus_node( $p ),
                       \%labellednodes, $graphviz );

      my $identifier = $p->acc . " " . $term->acc;
      unless( $relationships{ $identifier } ){
        ## for some reason graphviz assumes that a number is only a
        ## label, and not a node name which is irritating.
        my $node = "acc" . $term->acc;
        my $pnode = "acc" . $p->acc;
        my $apph = $term->apph;
        my $relation = $apph->get_relationships( {child=>$term, parent=>$p} );

        $graphviz->add_edge($pnode=>$node, label=>$relation->[0]->type);
        $relationships{ $identifier } = 1;
      }
    }
  }
  return $graphviz;
}

sub _graphviz_label{
  my $term = shift;
  my $graph = shift;
  my $labellednodes = shift;
  my $graphviz = shift;
  my $acc = $term->acc;
  my $node = "acc" . $term->acc;

  unless( $labellednodes->{$acc} ){
    $graphviz->add_node
      ( $node, label=>$term->name . " " . $term->acc );
    $labellednodes->{$acc} = 1;
  }
}

sub label_nodes_with_colour{
  my $graphviz = shift;
  my $terms = shift;
  my $colour = shift;

  foreach my $term (@$terms){
     my $node = "acc" . $term->acc;
    $graphviz->add_node
      ( $node,
        style=>"filled", color=>$colour );
   }
}

1;

