#!/usr/bin/perl -w

package GOGraph::GeneProduct::Collections::AllGOCollection;

=head1 NAME

  GOGraph::GeneProduct::Collections::AllGOCollection

=head1 DESCRIPTION

  This class returns all GeneProduct identifiers which are known to
  the GO database. See GOGraph::GeneProduct::Collections::SimpleCollection
  for further documentation

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};

use DBI;
use GOGraph::GOAppHandleFactory;
use GOGraph::GeneProduct::Collections::SimpleCollection;
use base qw{GOGraph::GeneProduct::Collections::SimpleCollection};

sub _fetch{
  my $self = shift;

  my $go_connection = GOGraph::GOAppHandleFactory::go_dbi_connection();

  ## fetch all the swiss prot proteins from the GO database.
  my $fetch_swiss=<<SQL;
select symbol from gene_product
  order by symbol
SQL

  my $go_sth = $go_connection->prepare( $fetch_swiss ) || die "$!";
  $go_sth->execute;

  my @swissprot_terms;
  while( my $swissprot_id_array = $go_sth->fetchrow_arrayref ){
    push( @swissprot_terms,
          $swissprot_id_array->[ 0 ] );
  }

  $go_sth->finish();
  $go_connection->disconnect();
  return \@swissprot_terms;
}


1;
