#!/usr/bin/perl -w

package GOGraph::GeneProduct::Collections::AllSwissprotCollection;


=head1 NAME

  GOGraph::GeneProduct::Collections::AllSwissprotCollection

=head1 DESCRIPTION

  This class returns all GeneProduct identifiers which are known to
  the GO database and which are sourced from Swissprot/Trembl. See
  GOGraph::GeneProduct::Collections::SimpleCollection for further
  documentation

  Note that what this class actually does is look for gene products
  with "sptr" in the dbref_name field. This value has already
  changed once during the life time of this class. It used to be "SP"
  The value comes directly from the association files and is subject
  to change.

  Hopefully in time the GO database will contain more direct provenance
  data, at which time this class will need to be changed.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};

use DBI;
use GOGraph::GOAppHandleFactory;
use GOGraph::GeneProduct::Collections::SimpleCollection;

use base qw{GOGraph::GeneProduct::Collections::SimpleCollection};


sub _fetch{
  my $self = shift;

  my $go_connection = GOGraph::GOAppHandleFactory::go_dbi_connection();

  ## fetch all the swiss prot proteins from the GO database.
  my $fetch_swiss=<<SQL;
select symbol from gene_product, dbxref
  where gene_product.dbxref_id=dbxref.id
  and dbxref.xref_dbname='sptr'
  order by symbol
SQL

  my $go_sth = $go_connection->prepare( $fetch_swiss ) || die "$!";
  $go_sth->execute;

  my @swissprot_terms;
  while( my $swissprot_id_array = $go_sth->fetchrow_arrayref ){
    push( @swissprot_terms,
          $swissprot_id_array->[ 0 ] );
  }

  $go_sth->finish();
  $go_connection->disconnect();
  return \@swissprot_terms;
}

1;
