#!/usr/bin/perl -w

package GOGraph::GeneProduct::Collections::SimpleCollection;

use strict;
use Carp;

=head1 NAME

  GOGraph::GeneProduct::Collections::SimpleCollection

=head1 SYNOPSIS

  This is an abstract class and can only be used through its subclasses.

  my $swiss = GOGraph::GeneProduct::Collections::AllSwissprotCollection->new;
  my $identifiers = $swiss->collection;

=head1 DESCRIPTION

  This abstract class defines a mechanism for getting collection of
  gene product accession numbers or identifiers, for use with the GO
  database.

=head2 collection

  Returns - a arrayref

  Returns the collection defined by this class.

=head2 _fetch

  A "protected" template method which sub classes should implement,
  and return the arrayref which defines the collection.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;

  my $self = {};
  bless $self, $class;
  $self->_initialize;
  return $self;
}

sub _initialize{
}

sub _fetch{
  croak "Calling _fetch on base class identifier.\n" .
    "This normally is an implementation error";
}

sub collection{
  my $self = shift;

  if( !defined $self->{collection} ){
    $self->{collection} = $self->_fetch;
  }

  return $self->{collection};
}

1;
