#!/usr/bin/perl -w

package GOGraph::GeneProduct::SemanticSimilarity::PairwiseAverageSimilarity;

=head1 NAME

  GOGraph::GeneProduct::SemanticSimilarity::PairwiseAverageSimilarity

=head1 DESCRIPTION

  This class combines similarities between two sets of GO terms, by
  calculating the semantic similarities for all pairwise combinations,
  and then averaging them. In this case it only takes similarities
  which are greater than 0, which probably makes sense. See the base
  class for more explanation.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};

use GOGraph::GeneProduct::SemanticSimilarity::CombinedSimilarity;
use base qw{GOGraph::GeneProduct::SemanticSimilarity::CombinedSimilarity};


sub semantic_similarity{
  my $self = shift;

  my $terms_one = shift;
  my $terms_two = shift;

  return $self->_pairwise_average( $terms_one, $terms_two );
}

sub _pairwise_average{
  my $self = shift;
  my $terms_one = shift;
  my $terms_two = shift;

  my $total_similarity = 0;
  my $no_comparisons = 0;

  foreach my $term_one ( @$terms_one ){
    foreach my $term_two ( @$terms_two ){
      my $sim = $self->{similarity}->similarity_between_terms
        ( $term_one, $term_two );
      if( $sim >= 0 ){
        $total_similarity +=
          $self->{similarity}->similarity_between_terms( $term_one, $term_two );
        $no_comparisons++;
      }
    }
  }

  if( $no_comparisons == 0 ){
    return -1;
  }
  return $total_similarity/$no_comparisons;
}



1;
