#!/usr/bin/perl -w

package GOGraph::GeneProduct::NeedleSimilarity;

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;

use Carp;

use File::Temp "tempfile";

use Bio::Factory::EMBOSS;
use Bio::SeqIO;
use Bio::AlignIO;


sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $self = {};
  my $emboss_factory = new Bio::Factory::EMBOSS();
  #$emboss_factory->verbose( 1 );
  $self->{factory} = $emboss_factory;

  bless $self, $class;
  return $self;
}

sub similarity{

  my $self = shift;
  my $seq1 = shift;
  my $seq2 = shift;

  my $factory = $self->{factory};
  my $needle = $factory->program('needle');

  ## Change this when I have it working
  my ($fh, $filename) = tempfile( UNLINK=>0 );

  $needle->run({ '-sequencea' => $seq1,
                 '-seqall'    => [ $seq2 ],
                 '-gapopen'   => '10.0',
                 '-gapextend' => '0.5',
                 '-outfile'   => $filename});# || croak $!;

  while( <$fh> ){
    #print "Line is $_ \n";
    ## Look for a line with like...
    ## "# Identity:     326/348 (93.7%)"
    ## and take out the percentage
    if( /^\#\s+Identity:\s+(\S+)\s+[(]\s*(\S+)[%][)]/ ){
      #print "Returning $2\n";
      return $2;
    }
  }
  #print "Didn't find any Identity value \n";
  return -1;
}


1;
