 #!/usr/bin/perl -w

package GOGraph::Gosst::GosstDatabaseResultsHandler;

use Exporter;
use strict;
use DBI;

use vars qw(@ISA);
use base qw(GOGraph::Statistics::CompareOutput Exporter);

use GOGraph::GeneProduct::SequenceSimilarity::SwissprotSequence;


my $instances = [];
my $results_id = 0;


sub _initialize{
  my $self = shift;
  my $comparitor_name = shift;

  if( defined $comparitor_name ){
    $self->{comparitor_name} = $comparitor_name;
    push @$instances, $self;

    $self->clear;
  }

  $self->{result} = {};
  $self->{swiss} = GOGraph::GeneProduct::SequenceSimilarity::SwissprotSequence->new();
}

sub process_result{
  my $self = shift;
  my $sequence_similarity = shift;
  my $semantic_similarity = shift;

  my $acc = shift;
  my $match_acc = shift;

  $self->{acc} = $acc;
  my $result = $self->{result};
  push @{$result->{$semantic_similarity}}, $match_acc;
}

sub clear{
  my $self = shift;
  $self->{result} = {};
}

sub commit{
  my $self = shift;
  my $insert_sth= $self->{insert_sth};
  my $cutoff = 0;

  foreach my $score ( reverse ( sort {$a <=> $b } ( keys %{$self->{result}} ) ) ){
    my $sequences = $self->{result}->{$score};
    foreach my $sequence ( @$sequences ){
      if ( $score <= 0 ){
        return;
      }
      if( $cutoff++ > 50 ){
        return;
      }

      my $seq;# = $self->{swiss}->sequence( $sequence );
      my $desc = "No Description";
      if( defined $seq ){
        $desc = substr( $seq->desc, 0, 60 );
      }
      $insert_sth->execute
        ( $results_id, $self->{acc}, $sequence, , $desc, $self->{constraint_id}, $score );
    }
  }
}

sub prepare_database{
  my $connection = DBI->connect( "dbi:mysql:database=gosst_results", "root", "" ) || die "Failed to connect $!";
  $connection->do( "delete from compare" );
  $connection->do( "delete from constraint_type" );

  my $insert
    = "insert into compare values( ?, ?, ?, ?, ?, ? )";

  my $insert_sth = $connection->prepare( $insert );

  my $constraint_id = 0;
  my $insert_constraint = $connection->prepare
    ( "insert into constraint_type values ( ?, ? )" );

  foreach my $inst( @$instances ){
    $inst->{connection} = $connection;
    $inst->{insert_sth} = $insert_sth;
    my $instance_comparitor_name = $inst->{comparitor_name};
    $insert_constraint->execute( $constraint_id, $instance_comparitor_name );
    $inst->{constraint_id} = $constraint_id;
    $constraint_id++;
  }

  $results_id = 0;

}

sub clear_results{
  foreach my $inst( @$instances ){
    $inst->clear;
  }
}

sub commit_results{
  foreach my $inst( @$instances ){
    $inst->commit;
  }
}


1;
