#!/usr/bin/perl -w

use strict;

use GOGraph::Statistics::GOCompare;
use GOGraph::Similarity::Factory;
use GOGraph::Gosst::GosstDatabaseResultsHandler;
use GOGraph::GeneProduct::Collections::AllSwissprotCollection;

my $comparitors = [];

push @$comparitors, GOGraph::Similarity::Factory::resnick_aspect_function();
push @$comparitors, GOGraph::Similarity::Factory::resnick_aspect_process();
push @$comparitors, GOGraph::Similarity::Factory::resnick_aspect_component();

# push @$comparitors, GOGraph::Similarity::Factory::lin_constraint
#   ( GOGraph::Similarity::Factory::constraint_all_relation_function_terms_evidence_above_9() );
# push  @$comparitors, GOGraph::Similarity::Factory::lin_constraint
#    ( GOGraph::Similarity::Factory::constraint_all_relation_process_terms_evidence_above_9() );
# push  @$comparitors, GOGraph::Similarity::Factory::lin_constraint
#   ( GOGraph::Similarity::Factory::constraint_all_relation_component_terms_evidence_above_9() );

#push @$comparitors, GOGraph::Similarity::Factory::jiang_constraint
#  ( GOGraph::Similarity::Factory::constraint_all_relation_function_terms_evidence_above_9() );
#push  @$comparitors, GOGraph::Similarity::Factory::jiang_constraint
#  ( GOGraph::Similarity::Factory::constraint_all_relation_process_terms_evidence_above_9() );
#push  @$comparitors, GOGraph::Similarity::Factory::jiang_constraint
#  ( GOGraph::Similarity::Factory::constraint_all_relation_component_terms_evidence_above_9() );

my $apph = GOGraph::GOAppHandleFactory::go_apph();


my $terms = {};
# my $root_term = $apph->get_root_term;
# $terms->{$root_term->acc} = $root_term;

# print "fetching all terms\n";
# get_all_terms( $terms, $root_term );

open (IN, "<terms.txt") || die "$!; Can't open terms file";
while( <IN> ){
  chomp;
  $terms->{$_} = $apph->get_term({acc=>$_});
}

foreach my $acc (keys %$terms){
  print "Term $acc " . $terms->{$acc}->acc . " " . $terms->{$acc}->public_acc .  "\n";
}

my @values_one = sort values %$terms;
my @values_two = sort values %$terms;

my $handles = {};
foreach my $comparitor (@$comparitors){
  $handles->{$comparitor} = FileHandle->new( "> go_compare_" . $comparitor->constraint_name . ".dat" ) || croak $!;
}


foreach my $term_one (@values_one ){
  print "Comparing " . $term_one->name ."\n";
  foreach my $term_two (@values_two){
    compare_terms( $term_one, $term_two );
  }
}



sub get_all_terms{
  my $terms = shift;
  my $term = shift;

  ## fetch all child relationships
  my $child_terms = $apph->get_child_terms($term);
  foreach my $child_term (@$child_terms){

    if( !defined( $terms->{$child_term->acc} ) ){
      $terms->{$child_term->acc} = $child_term;
      get_all_terms( $terms, $child_term );
    }
  }
}

sub compare_terms{
  my $term_one = shift;
  my $term_two = shift;

  foreach my $comparitor (@$comparitors){
    my $sim = $comparitor->similarity_between_terms( $term_one, $term_two );
    my $name = $comparitor->constraint_name();

    $handles->{$comparitor}->print( $term_one->acc . "\t" . $term_two->acc . "\t" . $sim . "\n" );
  }
}


