#!/usr/bin/perl -w




=head1 NAME

  similarity_search

=head1 DESCRIPTION

  This provides a simple search tool based over the semantic similarity
  library. It current works over swissprot, and simply compares the query
  protein with every other protein.

  This is remarkably slow the first time around, say 10 minutes per search.
  However by making extensive use of memory caching, things are quicker
  the next time around (by more than an order of magnitude). 

=cut

use strict;


# use GOGraph::Statistics::BlastDispatcher;
use GOGraph::GeneProduct::Collections::AllSwissprotCollection;
use GOGraph::Statistics::GOCompare;
use GOGraph::Similarity::Factory;
use GOGraph::Statistics::PrintSortedResults;
$| = 1;
my $comparitors = [];

push @$comparitors, GOGraph::Similarity::Factory::resnick_aspect_function_good();
push @$comparitors, GOGraph::Similarity::Factory::resnick_aspect_process_good();
push @$comparitors, GOGraph::Similarity::Factory::resnick_aspect_component_good();

# push @$comparitors, GOGraph::Similarity::Factory::lin_constraint
#   ( GOGraph::Similarity::Factory::constraint_all_relation_function_terms_evidence_above_9() );
# push  @$comparitors, GOGraph::Similarity::Factory::lin_constraint
#    ( GOGraph::Similarity::Factory::constraint_all_relation_process_terms_evidence_above_9() );
# push  @$comparitors, GOGraph::Similarity::Factory::lin_constraint
#   ( GOGraph::Similarity::Factory::constraint_all_relation_component_terms_evidence_above_9() );

#push @$comparitors, GOGraph::Similarity::Factory::jiang_constraint
#  ( GOGraph::Similarity::Factory::constraint_all_relation_function_terms_evidence_above_9() );
#push  @$comparitors, GOGraph::Similarity::Factory::jiang_constraint
#  ( GOGraph::Similarity::Factory::constraint_all_relation_process_terms_evidence_above_9() );
#push  @$comparitors, GOGraph::Similarity::Factory::jiang_constraint
#  ( GOGraph::Similarity::Factory::constraint_all_relation_component_terms_evidence_above_9() );



my $printout = GOGraph::Statistics::PrintSortedResults->new();

my $goaspect = GOGraph::Statistics::GOCompare->new
  ( $comparitors, $printout );


use GOGraph::GeneProduct::Collections::AllSGDCollection;
my $search_proteins =
  GOGraph::GeneProduct::Collections::AllSGDCollection->new->collection;

my @blah = @$search_proteins;
$#blah = 150;
$search_proteins = \@blah;
print "size is " . scalar @$search_proteins . "\n";


my $query_protein;
print "PROTEIN?\n";

while( <> ){
  chomp;
  $query_protein = $_;

  print "Searching with $query_protein \n";
  my $start = time;
  foreach my $search ( @$search_proteins ){
    print $search . "\n";
    $goaspect->compare( $query_protein, $search );
  }

  print "\n";

  $printout->print_results;
  $printout->clear_results;

  my $time = time - $start;
  print "Search took $time seconds \n";

  print "PROTEIN?\n";
}

print "Complete\n";

