#!/usr/bin/perl -w


package GOGraph::Similarity::Factory;


=head1 NAME

  GOGraph::Similarity::Factory

=head1 DESCRIPTION

  This module is just a standard place to put factory code.
  Its a little bit messy, but I can't think of a better way of doing it
  at the moment. The problem is that the Similarity packages work by
  combinging together lots of objects in various orders, which produce
  the different combinations of similarity measures. The practical upshot
  of this is that driver code is getting bigger and bigger as time goes on.
  At least if I put it all in here, the mess is in one place only.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;

use GOGraph::Crawl::AcceptAll;
use GOGraph::Crawl::AcceptSubsumption;
use GOGraph::Crawl::AcceptNone;
use GOGraph::Crawl::CrawlToRoot;
use GOGraph::Similarity::InformationContent::Base::AcceptAspect;
use GOGraph::Similarity::InformationContent::Base::AcceptEvidence;
use GOGraph::Similarity::InformationContent::Base::AcceptOnlyEvidence;
use GOGraph::Similarity::InformationContent::Base::AcceptMultiplexer;
use GOGraph::Similarity::InformationContent::Base::Constraints;
use GOGraph::Similarity::InformationContent::Resnick99::ResnickSimilarity;
use GOGraph::Similarity::InformationContent::Lin98::LinSimilarity;
use GOGraph::Similarity::InformationContent::Jiang97::JiangSimilarity;


### Accept term factory code.....
sub accept_term_aspect{
  my $aspect = shift;
  return GOGraph::Similarity::InformationContent::Base::AcceptAspect->new( $aspect );
}

sub accept_term_evidence_n{
  my $evidence_threshold = shift;
  return GOGraph::Similarity::InformationContent::Base::AcceptEvidence
    ->new( $evidence_threshold );
}

sub accept_term_multi{
  my $acceptors = shift;
  return GOGraph::Similarity::InformationContent::Base::AcceptMultiplexer
    ->new( $acceptors );
}

sub accept_term_evidence_over_9{
  return accept_term_evidence_n( 0.9 );
}

sub accept_term_evidence_only{
  my $evidence = shift;
  return GOGraph::Similarity::InformationContent::Base::AcceptOnlyEvidence
    ->new( $evidence );
}

### Accept relationship factory....
sub accept_relationship_all{
  return GOGraph::Crawl::AcceptAll->new();
}

sub accept_relationship_subsumption{
  return GOGraph::Crawl::AcceptSubsumption->new();
}

sub accept_relationship_none{
  return GOGraph::Crawl::AcceptNone->new();
}

### Crawl to root factory...
sub crawl_to_root{
  my $accept_relationship = shift;
  ## can be undef, in which case will take Gene Ontology node
  my $root_node = shift;
  return GOGraph::Crawl::CrawlToRoot->new( $accept_relationship, $root_node );
}

sub crawl_to_root_all{
  ## can be null in which case Gene Ontology node
  my $root_node = shift;
  return crawl_to_root
    ( accept_relationship_all(), $root_node );
}


sub crawl_to_root_none{
  my $root_none = shift;
  return crawl_to_root
    ( accept_relationship_none(), $root_none );
}

## actual crawlers I might want to use...
sub crawl_to_root_subsumption_root{
  return crawl_to_root
    ( accept_relationship_subsumption() )
  }

sub crawl_to_root_all_root{
  return crawl_to_root_all();
}

sub crawl_to_root_all_function_root{
  return crawl_to_root_all( 3674 );
}

sub crawl_to_root_all_process_root{
  return crawl_to_root_all( 8150 );
}

sub crawl_to_root_all_component_root{
  return crawl_to_root_all( 5575 );
}


sub crawl_to_root_none_function{
  return crawl_to_root_none( 3674 );
}

sub crawl_to_root_subsumption_function{
  return crawl_to_root
    ( accept_relationship_subsumption(), 3674 );
}

### Constraint factory....
sub constraint{
  my $crawler = shift;
  ## can be null in which case accept any
  my $accept_term = shift;
  return
    GOGraph::Similarity::InformationContent::Base::Constraints->new( $crawler, $accept_term );
}


sub constraint_all_relation_all_term{
  return constraint( crawl_to_root_all_root() );
}

sub constraint_subsumption_relationship_all_term{
  return constraint( crawl_to_root_subsumption_root() );
}

sub constraint_all_relation_function_terms{
  return constraint
    ( crawl_to_root_all_function_root(),
      accept_term_aspect( "function" ) );
}

sub constraint_all_relation_component_terms{
  return constraint
    ( crawl_to_root_all_component_root(),
      accept_term_aspect( "component" ) );
}

sub constraint_all_relation_process_terms{
  return constraint
    ( crawl_to_root_all_process_root(),
      accept_term_aspect( "process" ) );
}

sub constraint_all_relation_function_terms_evidence_above_9{
  return constraint
    ( crawl_to_root_all_function_root(),
      accept_term_multi
      ( [ accept_term_aspect( "function" ),
          accept_term_evidence_over_9() ] ) );
}


sub constraint_all_relation_process_terms_evidence_above_9{
  return constraint
    ( crawl_to_root_all_process_root(),
      accept_term_multi
      ( [ accept_term_aspect( "process" ),
          accept_term_evidence_over_9() ] ) );
}

sub constraint_all_relation_component_terms_evidence_above_9{
  return constraint
    ( crawl_to_root_all_component_root(),
      accept_term_multi
      ( [ accept_term_aspect( "component" ),
          accept_term_evidence_over_9() ] ) );
}

sub constraint_none_relation_function_terms{
  return constraint
    ( crawl_to_root_none_function(), accept_term_aspect( "function" ) );
}

sub constraint_subsumption_relation_function_terms{
  return constraint
    ( crawl_to_root_subsumption_function(),
      accept_term_aspect( "function" ) );
}


sub constraint_none_relation_function_terms_evidence_above_9{
  return constraint
    ( crawl_to_root_none_function(),
      accept_term_multi
      (  [ accept_term_aspect( "function" ),
           accept_term_evidence_over_9() ] ) );
}

sub constraint_subsumption_relation_function_terms_evidence_above_9{
  return constraint
    ( crawl_to_root_subsumption_function(),
      accept_term_multi
      (  [ accept_term_aspect( "function" ),
           accept_term_evidence_over_9() ] ) );
}


sub constraint_all_relation_function_terms_evidence{
  my $evidence = shift;
  return constraint
    ( crawl_to_root_all_function_root(),
      accept_term_multi
      ( [ accept_term_aspect( "function" ),
          accept_term_evidence_only( $evidence ) ] ) );
}

sub constraints_all_relation_function_terms_evidence_array_reference{
  my @constraints;
  foreach my $evidence ( qw{IMP IGI IPI ISS IDA IEP IEA TAS NAS NR} ) {
    push @constraints,
      constraint_all_relation_function_terms_evidence( $evidence );
  }
  return \@constraints;
}

sub constraints_array_reference{
  my $constraints = 
    [ constraint_all_relation_all_term(),
      constraint_subsumption_relationship_all_term(),
      constraint_all_relation_function_terms(),
      constraint_all_relation_component_terms(),
      constraint_all_relation_process_terms(),
      constraint_all_relation_function_terms_evidence_above_9(),
      constraint_all_relation_process_terms_evidence_above_9(),
      constraint_all_relation_component_terms_evidence_above_9(),
      constraint_none_relation_function_terms(),
      constraint_subsumption_relation_function_terms(),
      constraint_none_relation_function_terms_evidence_above_9,
      constraint_subsumption_relation_function_terms_evidence_above_9
    ];

  push @$constraints, @{constraints_all_relation_function_terms_evidence_array_reference() };
  return $constraints;
}



#### Here are all the resnick end methods.
sub resnick_constraint{
  my $constraints = shift;
  return GOGraph::Similarity::InformationContent::Resnick99::ResnickSimilarity
    ->new( $constraints );
}


sub resnick_sim_all{
  return resnick_constraint( constraint_all_relation_all_term() );
}

sub resnick_sim_subsumption{
  return resnick_constraint( constraint_subsumption_relationship_all_term() );
}


sub resnick_aspect_function{
  return resnick_constraint( constraint_all_relation_function_terms() );
}

sub resnick_aspect_process{
  return resnick_constraint( constraint_all_relation_process_terms() );
}

sub resnick_aspect_component{
  return resnick_constraint( constraint_all_relation_component_terms() );
}

sub resnick_aspect_function_good{
  return resnick_constraint( constraint_all_relation_function_terms_evidence_above_9() );
}

sub resnick_aspect_process_good{
  return resnick_constraint( constraint_all_relation_process_terms_evidence_above_9() );
}

sub resnick_aspect_component_good{
  return resnick_constraint( constraint_all_relation_component_terms_evidence_above_9() );
}

sub resnick_similarities_evidence_array_reference{
  my $constraints = constraints_all_relation_function_terms_evidence_array_reference();
  my @sim;

  foreach my $constraint ( @$constraints ) {
    push @sim, resnick_constraint
      ( $constraint );
  }

  return \@sim;
}

sub resnick_similarities_array_reference{
  my $constraints = constraints_array_reference();
  my @sim;

  foreach my $constraint ( @$constraints ) {
    push @sim, resnick_constraint
      ( $constraint );
  }

  return \@sim;
}

###Lin similiarities
sub lin_constraint{
  my $constraints = shift;
  return GOGraph::Similarity::InformationContent::Lin98::LinSimilarity
    ->new( $constraints );
}

sub lin_similarities_array_reference{
  my $constraints = constraints_array_reference();
  my @sim;

  foreach my $constraint ( @$constraints ) {
    push @sim, lin_constraint
      ( $constraint );
  }

  return \@sim;
}

###Jiang similarities
sub jiang_constraint{
  my $constraints = shift;
  return GOGraph::Similarity::InformationContent::Jiang97::JiangSimilarity
    ->new( $constraints );
}

sub jiang_similarities_array_reference{
  my $constraints = constraints_array_reference();
  my @sim;

  foreach my $constraint ( @$constraints ) {
    push @sim, jiang_constraint
      ( $constraint );
  }

  return \@sim;
}


sub all_similiarities_array_reference{
  my $comparitors =
    resnick_similarities_array_reference();

  push( @$comparitors,
        @{lin_similarities_array_reference()}
      );

  push( @$comparitors,
        @{jiang_similarities_array_reference()}
      );

  return $comparitors
}

1;
