#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::Accept;


=head1 NAME

  GOGraph::Similarity::InformationContent::Base::Accept

=head1 SYNOPSIS

  This is (mostly) an abstract class. This implementation just returns
  true for everything.

  my $cons = GOGraph::Similarity::InformationContent::Base::Constraints->
  new( $crawler, GOGraph::Similarity::InformationContent::Base::Accept->new );

  my $similarity =
  GOGraph::Similarity::InformationContent::Resnick99::ResnickSimilarity
    ->new( $cons );

=head1 DESCRIPTION

  This class defines whether we should accept a Term or not. There are
  a number of reasons why we might not. If for instance the Term is
  from a different aspect, than the one used to calculate information
  content, then there is little point in going further. Also we might
  accept terms with only a certain type evidence code.

=head2 accept

  Args - GO::Model::Term - is this term acceptable or not.
  Returns - true or false!

  Defines an interface for ensuring that a term is acceptable for
  Similarity testing.

=head2 name

  Returns - constraint string

  String which is used to key the information content precalculation
  database.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  bless $self, $class;
  return $self;
}

sub accept{
  return 1;
}

sub name{
  return "allterms";
}


1;
