#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::AcceptAspect;

use strict;
use Carp;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::AcceptAspect;

=head1 DESCRIPTION

  See GOGraph::Similarity::InformationContent::Base::Accept; for
  fuller description. In this case we accept only terms of a given
  aspect, as defined by the GO::Model::Term->type return value. As a
  backwards compatibility step, we also allow "molecular_function",
  "biological_process", and "cellular_component" as synonyms.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut



my $valid_aspects = {"function"=>"function",
                     "process"=>"process",
                     "component"=>"component",

                     "molecular_function"=>"function",
                     "biological_process"=>"process",
                     "cellular_component"=>"component"};


sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $aspect = shift;

  ## stop doing this as it seems to break things...
  ## normalise over different naming conventions.
  # if( $aspect eq "function" ){
#     $aspect = "molecular_function";
#   }

#   if( $aspect eq "process" ){
#     $aspect = "biological_process";
#   }

#   if( $aspect eq "component" ){
#     $aspect = "cellular_component";
#   }

  my $self = {};
  bless $self, $class;

  $self->{aspect} = $self->normalise_aspect( $aspect );
  return $self;
}

sub normalise_aspect{
  my $self = shift;
  my $aspect = shift;

  unless( defined $valid_aspects->{$aspect} ){
    croak "There is a problem. The GOGraph software makes use of information\n" .
      "provided by the GO database on the aspect of a given term.\n" .
        "This is identified by a string value which appears to change sometimes\n" .
          "which results in terms being ignored (as they appear to be in the wrong aspect\n" .
            "where this should not be the case. This appears to have happened here with\n" .
              "as a term is returning as aspect: " . $aspect . "\n" .
                "The practical upshot of this is that you are likely to recieve a lot of similarities\n" .
                  "of -1, when they shouldn't be.\n";

  }

  return $valid_aspects->{$aspect};
}

sub accept{
  my $self = shift;
  my $term = shift;

  #print "aspect is " . $term->type . ($term->type eq $self->{aspect}) . "\n";
  return (  $self->normalise_aspect( $term->type ) eq $self->{aspect} );
}


sub name{
  my $self = shift;
  return  $self->{aspect} . "terms";
}

1;
