#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::AcceptMultiplexer;

use strict;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::AcceptMultiplexer;

=head1 DESCRIPTION

  See GOGraph::Similarity::InformationContent::Base::Accept; for
  fuller description. This class allows the combination of several
  other Accept objects, and returns true iff all of these return true.

=head2 new

  Args - an arrayref of Accept objects.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $accept = shift;

  my $self = { acceptor=>$accept };
  bless $self, $class;
  return $self;
}

sub accept{
  my $self = shift;
  my $term = shift;

  foreach my $accept ( @{$self->{acceptor}} ){
    if( ! $accept->accept( $term ) ){
      return 0;
    }
  }

  return 1;
}

sub name{
  my $self = shift;

  if( ! defined ($self->{name}) ){
    my @names;
    foreach my $accept ( @{$self->{acceptor}} ){
      push  @names, $accept->name;
    }

    @names = sort( @names );

    my $fullname = "acceptmulti";
    foreach my $name ( @names ){
      $fullname .= "$name"
    }
    $self->{name} = $fullname;
  }
  return $self->{name};
}


1;
