#!/usr/bin/perl -w


package GOGraph::Similarity::InformationContent::Base::Constraints;

use strict;

use GOGraph::Similarity::InformationContent::Base::Accept;

=head1 NAME

  GOGraph::Similarity::InfromationContent::Base::Constraints

=head1 DESCRIPTION

  This class defines constraints which are to be used in calculating
  the similarity values. These constraints will be a variety of Objects
  of defined types.

  These constraints need to be the known during the calculation of the
  similarity database, and during its look up. By encapsulating them here
  we can encapsulate the way this is mapped to the database schema.

  GOGraph::Crawl::CrawlToRoot :- How a term is traced back to the GeneOntology
  root, affects the number of times parental terms occur.

  Next we have the root to which we are crawling, and finally we have
  an GOGraph::Similarity::InformationContent::Base::Accept which tells
  us whether a term is good or not

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $crawler = shift;
  my $accept_term = shift || GOGraph::Similarity::InformationContent::Base::Accept->new();

  my $self =
    {
     crawler=>$crawler,
     accept_term=>$accept_term
    };

  bless $self, $class;
  return $self;
}

sub get_crawler{
  my $self = shift;
  return $self->{crawler};
}

sub get_accept_term{
  my $self = shift;
  return $self->{accept_term};
}

sub _get_description_from_constraints{
  my $self = shift;

  my $root_name;
  ## this gives us the root of the hierarchy.
  my $root = $self->{crawler}->get_root_acc;

  return $self->{crawler}->get_accept->name() . $self->{accept_term}->name .  $root;
}

1;
