#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::Occurrence;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::Occurrence;

=head1 DESCRIPTION

  An abstract class for determinging the information content for each
  of the GO terms. Instances of this class are generally created with
  the OccurrenceFactory class.

=head2 fetch_max_occurrences

  Args- A string representing the constraints on the occurence
  calculation.
  Return - an integer representing the max number of occurences of
  any terms for this set of constraints.

=head2 fetch_occurrences

  Args- A string representing the constraints on the occurence calculation.
  And a go accn number.
  Return- an integer representing the number of occurences for term in
  the database.

=head2 create_instance

  Return- a new instance of this object.

  This factory method is probably what you want to use to return a get a
  concrete instance of this class.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut


use strict;


sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;

  my $self = {};
  bless $self, $class;

  $self->_initialize(@_);
  return $self;
}

sub _initialize{
}

=head2 fetch_probability
  Args- A string representsenting the constraints on the occurence
  calculation. And a go accn number
  Return- The probablility of this occuring.

=cut
sub fetch_probability{
  my $self = shift;
  my $constraints = shift;
  my $goaccn = shift;

  my $occur = $self->fetch_occurrences( $constraints, $goaccn );
  my $max = $self->fetch_max_occurrences( $constraints );

  unless( $occur > 0 && $max > 0 ){
    return -1;
  }
  my $probability = $occur / $max;
  return $probability;
}


1;
