#!/usr/bin/perl -w

package GOGraph::Statistics::BlastDispatcher;

use GOGraph::GeneProduct::Collections::AllSwissprotCollection;
use GOGraph::GeneProduct::SequenceSimilarity::BlastSwissprot;


use strict;



sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;
  ## this is a Swissprot.pm object....
  my $dispatch = shift;
  my $swissprot_collection = shift ||
    GOGraph::GeneProduct::Collections::AllSwissprotCollection->new();
  my $self = {};

  $self->{dispatch} = $dispatch;
  $self->{swissprot_collection} = $swissprot_collection;
  bless $self, $class;
  return $self;
}



## does blast on each of the elements in a random order.
sub straight_blast_hits{
  my $self = shift;
  my $fh = shift;

  my @swissprot_accessions = @{ $self->{swissprot_collection}->collection };
  my @retn;

  my $dispatch = $self->{dispatch};
  my $length = @swissprot_accessions;

  my $blast = GOGraph::GeneProduct::SequenceSimilarity::BlastSwissprot->new();

  #$self->_fisher_yates_shuffle( \@swissprot_accessions );

  my $proteins_analysed = 0;
  my $total_seconds = 0;
  my $last_start = time;

  for( my $i = 0;
       $i < $length;
       $i++ ){
    my $acc = $swissprot_accessions[$i];

    ## metrics
    my $this_start = time;
    my $last_seconds = $this_start - $last_start;
    $total_seconds = $total_seconds + $last_seconds;
    my $average_seconds =
      eval{$total_seconds / $proteins_analysed++} || 0;
    print "\n$proteins_analysed: Blasting $acc. Metrics-" .
      " last $last_seconds Average $average_seconds Total $total_seconds\n";
    $last_start = $this_start;
    my $blast_report = $blast->blast_swissprot_acc( $acc );

    if( defined $blast_report ){
      while( my $subject = $blast_report->nextSbjct ){
        if( $subject->name =~ /sp[|]\w+[|](\w+)/ ){
          my $match_acc = $1;
          print ".";
          #if( $proteins_analysed % 100 == 0 ){
          #  print "Proteins analysed $proteins_analysed\n";
          #}
          my $hsp = $subject->nextHSP;
          my $bit_score = $hsp->bits;
          my $sequence_similarity = log( $bit_score );
          $dispatch->compare( $acc, $match_acc, $sequence_similarity );
        }
      }
    }
  }
}

sub _fisher_yates_shuffle{
  ## stolen from the perl cook book.
  my $self = shift;
  my $array = shift;

  my $i;
  for( $i = @$array;
       --$i;
     ){
    my $j = int rand ( $i + 1 );
    next if $i == $j;
    @$array[$i,$j] = @$array[$j, $i];
  }
}

sub random_blast_hits{
  my $self = shift;
  my $number = shift;
  my $fh = shift;

  my @swissprot_accessions = @{ $self->{swissprot_collection}->collection };
  my @retn;

  my $dispatch = $self->{dispatch};
  my $length = @swissprot_accessions;

  my $blast = GOGraph::GeneProduct::BlastSwissprot->new();


  for( my $i = 0;
       $i < $number;
       $i++ ){
    my $acc = @swissprot_accessions[ rand( $length ) ];

    #print "$i: Blasting $acc\n";
    my $blast_report = $blast->blast_swissprot_acc( $acc );

    if( defined $blast_report ){
      while( my $subject = $blast_report->nextSbjct ){
        if( $subject->name =~ /sp[|]\w+[|](\w+)/ ){
          my $match_acc = $1;

          $dispatch->compare( $acc, $match_acc, $subject );
        }
      }
    }
  }
}


1;
