#!/usr/bin/perl -w

=head1 NAME

  CharacteriseEvidence

=head1 DESCRIPTION

  Characterises the number of different evidence codes present
  in the GO database.

  My original interest in this, was that to find which different
  types of evidence code where used in SWISSPROT.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use GOGraph::GOAppHandleFactory;
use GOGraph::GeneProduct::Collections::AllSwissprotCollection;
use GOGraph::GeneProduct::Collections::AllGOCollection;

$| = 1;
my $apph = GOGraph::GOAppHandleFactory::go_apph();

my $evidences = {};

my $ident = GOGraph::GeneProduct::Collections::AllSwissprotCollection->new()
  ->collection();


print "Analysing swissprot for evidence tags\n";
$evidences->{swissprot} = analyse( $ident );

$ident = GOGraph::GeneProduct::Collections::AllGOCollection->
  new()->collection();

print "Analysing everything for evidence tags\n";
$evidences->{all} = analyse( $ident );

Dumpvalue->new->dumpValue( $evidences );

sub analyse{

  my $ident = shift;

  # shorten everything
#  my @blah = @$ident;
#  $#blah = 1000;
#  $ident = \@blah;

  my $evidences = {};

  foreach my $accession ( @$ident ){
    my $terms = $apph->get_terms({'product'=>$accession});
    foreach my $term ( @$terms ){
      my $associations = $term->selected_association_list;
      foreach my $association (@$associations){
        my $evidence_list = $association->evidence_list();
        foreach my $evidence (@$evidence_list){
          $evidences->{$evidence->code()}++;
        }
      }
    }
    print ".";
  }

  print "\n";

  use Dumpvalue;

  Dumpvalue->new->dumpValue( $evidences );
  return $evidences;
}
