#!/usr/bin/perl -w

=head1 NAME

  CharacteriseEvidenceNumber

=head1 DESCRIPTION

  This script analyses all of the associations in first SWISS-PROT and
  secondly all of the GO database, and counts how gene products have
  no annotation, how many one annotation and so on. The kind of associations
  that are accepted are defined using the constraint sets from the
  information content sets.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut
  
use strict;
use GOGraph::GOAppHandleFactory;
use GOGraph::GeneProduct::Collections::AllSwissprotCollection;
use GOGraph::GeneProduct::Collections::AllGOCollection;
use GOGraph::Similarity::InformationContent::Base::AcceptEvidence;
use GOGraph::Similarity::Factory;

$| = 1;

my $apph = GOGraph::GOAppHandleFactory::go_apph();

my $evidences = {};

my $ident = GOGraph::GeneProduct::Collections::AllSwissprotCollection->new()
  ->collection();


my $acceptor = GOGraph::Similarity::Factory::accept_term_multi
  ( [ GOGraph::Similarity::Factory::accept_term_aspect( "function" ),
      GOGraph::Similarity::Factory::accept_term_evidence_over_9() ] );

print "Analysing swissprot for evidence number\n";
$evidences->{swissprot} = analyse( $ident );

$ident = GOGraph::GeneProduct::Collections::AllGOCollection->new()->collection();

print "Analysing everything for evidence number\n";
$evidences->{all} = analyse( $ident );

Dumpvalue->new->dumpValue( $evidences );

sub analyse{

  my $ident = shift;

  #shorten everything
  #my @blah = @$ident;
  #$#blah = 500;
  #$ident = \@blah;

  my $numbers = [];

  foreach my $accession ( @$ident ){
    my $terms = $apph->get_terms({'product'=>$accession});

    my $number_of_terms = 0;
    foreach my $term ( @$terms ){
      if( $acceptor->accept( $term ) ){
        $number_of_terms++;
      }
    }
    $numbers->[ $number_of_terms ] += 1;
    if( $number_of_terms > 1 ){
      print "$accession\n";
    }
    #print ".";
  }
  print "\n";

  use Dumpvalue;

  Dumpvalue->new->dumpValue( $numbers );

  return $numbers;
}
