#!/usr/bin/perl -w

=head1 NAME

  CharacteriseSharedRelationships

=head1 DESCRIPTION

  This class tells you how many terms have both isa and partof
  parents how many just isa, just partof, and how many no
  parents.

  Can't remember why I wanted to know this.

  Interestingly qauite a large number (50 or so) of terms actually
  have no parents. At first sight there should only be one
  (GeneOntology), but the GO database actually also creates partof,
  and isa terms (rather than relationships), which makes three. The
  rest all appear to be type f errors.

=cut

use strict;

use GOGraph::GOAppHandleFactory;


my $apph = GOGraph::GOAppHandleFactory::go_apph();

my $terms = $apph->get_terms({search=>"*"});

my $part_of_only = 0;
my $isa_only = 0;
my $none = 0;

my $both = 0;


foreach my $term ( @$terms ){

  my $relationships = $apph->get_relationships({child=>$term});

  my $isa = 0;
  my $part = 0;

  foreach my $relationship( @$relationships ){
    if( $relationship->is_inheritance ){
      $isa = 1;
      if( $part ){
        $both++;
        last;
      }
    }
    if( $relationship->is_partof ){
      $part = 1;
      if( $isa ){
        $both++;
        last;
      }
    }
  }
  if( $isa ){
    $isa_only++;
  }
  if( $part ){
    $part_of_only++;
  }
  if( ! $isa && ! $part ){
    $none++;
  }
}


print "both $both isa $isa_only part $part_of_only none $none\n";
